package com.jln.ld.map;

import java.util.ArrayList;
import java.util.List;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.Pixmap;
import com.badlogic.gdx.graphics.Texture;
import com.jln.ld.GameConstant;
import com.jln.ld.GameEntity;
import com.jln.ld.GamePoint;
import com.jln.ld.GameShared;
import com.jln.ld.menus.GameMenu;

public class GameMap extends GameEntity{

	private int mapCode;
	private Texture backgroundTexture;
	private Texture maskTexture;
	private int[][] collision;
	private List<MapCloud> clouds, frontClouds;
	private List<AnimatedMapItem> mapItems;

	/**
	 * Constructeur
	 * @param mapCode
	 */
	public GameMap(int mapCode){
		this.mapCode = mapCode;
		this.initRessources();
	}
	
	/**
	 * Initialise les ressources de la map
	 */
	public void initRessources(){
		if(mapCode == 1){
			backgroundTexture = GameConstant.MAP01_BACK;
			maskTexture = GameConstant.MAP01_MASK;
			
			Pixmap pixmap;
			Color color;
			
			pixmap = new Pixmap(Gdx.files.internal("maps/map01_mask.png"));
			color = new Color();
			int val;
			
			collision = new int[36][64];
			for(int i=0;i<36;i++){
				for(int j=0;j<64;j++){
					val = pixmap.getPixel(j, i);
				    Color.rgba8888ToColor(color, val);
				    int r = (int)(color.r * 255f);
				    int g = (int)(color.g * 255f);
				    int b = (int)(color.b * 255f);
				    if(r==255 && g==0 && b==0){
				    	collision[i][j] = 2;
				    }else if(r==0 && g==0 && b==0){
				    	collision[i][j] = 1;
				    }else if(r==200 && g==200 && b==200){
				    	collision[i][j] = 3;
				    }else{
				    	collision[i][j] = 0;
				    }
//				    System.out.print(collision[i][j]);
				}
//				System.out.println();
			}

			this.clouds = new ArrayList<MapCloud>();
			this.frontClouds = new ArrayList<MapCloud>();
			for(int i=0;i<6;i++){
				this.clouds.add(new MapCloud(false));
			}
			for(int i=0;i<3;i++){
				this.frontClouds.add(new MapCloud(true));
			}
			this.mapItems = new ArrayList<AnimatedMapItem>();
			this.mapItems.add(new AnimatedMapItem(AnimatedMapItem.ITEM_FLOWER,140,260));
			this.mapItems.add(new AnimatedMapItem(AnimatedMapItem.ITEM_FLOWER,120,120));
			this.mapItems.add(new AnimatedMapItem(AnimatedMapItem.ITEM_FLOWER,270,230));
			this.mapItems.add(new AnimatedMapItem(AnimatedMapItem.ITEM_FLOWER,430,110));
			this.mapItems.add(new AnimatedMapItem(AnimatedMapItem.ITEM_FLOWER,520,220));
			this.mapItems.add(new AnimatedMapItem(AnimatedMapItem.ITEM_FLOWER,180,250));
			this.mapItems.add(new AnimatedMapItem(AnimatedMapItem.ITEM_FLOWER,310,129));
			this.mapItems.add(new AnimatedMapItem(AnimatedMapItem.ITEM_FLOWER,290,239));
			this.mapItems.add(new AnimatedMapItem(AnimatedMapItem.ITEM_FLOWER,410,125));
			this.mapItems.add(new AnimatedMapItem(AnimatedMapItem.ITEM_FLOWER,500,240));

		}
	}
	
	/**
	 * 
	 * @param x
	 * @param y
	 * @return le type de terrain : 0 = passable, 1 = bloqu, 2=vide
	 */
	public int getMapState(float x, float y){
		int i = (int)y;
		int j = (int)x;
		i = i/10;
		j = j/10;
		if(i>35){
			i=35;
		}
		if(j<1){
			j=1;
		}
		if(j>63){
			j=63;
		}
		if(i<0){
			i = 0;
		}
	   return collision[35-(i)][j-1];
	}
	/**
	 * Maj la map
	 * @param game
	 */
	public void update(GameMenu game){
		for(MapCloud c : clouds){
			c.update(game);
		}
		for(MapCloud c : frontClouds){
			c.update(game);
		}
		for(AnimatedMapItem  a: mapItems){
			a.update(game);
		}
	}
	
	/**
	 * 
	 * @return
	 */
	public GamePoint getRandomTargetPoint(){
		GamePoint p = new GamePoint((float)Math.random()*640,(float)Math.random()*360);
		while(getMapState(p.getX(),p.getY())!=0){
			p.setX((float)Math.random()*GameConstant.GAME_WIDTH);
			p.setY((float)Math.random()*GameConstant.GAME_HEIGHT);
		}
		return p;
	}
	
	/**
	 * Dessine l'arriere plan
	 * @param game
	 */
	public void renderBackground(GameMenu game){
		GameShared.instance.gameBatch.draw(GameConstant.MAP_SKY, 0,0);
		GameShared.instance.gameBatch.draw(GameConstant.MAP_MOUNTAINS, 0,0);
		for(MapCloud c : clouds){
			c.render(game);
		}
	}
	
	/**
	 * Dessine la map
	 * @param game
	 */
	public void render(GameMenu game){
		if(backgroundTexture != null){
			GameShared.instance.gameBatch.draw(backgroundTexture, 0,0);
			for(AnimatedMapItem  a: mapItems){
				a.render(game);
			}
//			GameShared.instance.gameBatch.draw(maskTexture, 0,0);
		}
	}
	
	/**
	 * Dessine la map
	 * @param game
	 */
	public void renderFront(GameMenu game){
		for(MapCloud c : frontClouds){
			c.render(game);
		}
	}
}
